
pro diag,ps=ps

; omega_mat/omega_lam diagram, with online solution of Friedmann equation;
; omega_rad neglected

  if keyword_set(ps) then begin
    set_plot,'ps'
    device,file='diag.ps',/isolatin1,xsize=19,ysize=24,/encapsulated
    chars=1.0
    isotropic=1
    aspect=1
  endif else begin
    Device, Window_State=windowstate
    if windowstate[0] eq 0 then window,0
    if windowstate[1] eq 0 then window,1
    chars=1.5
    isotropic=0
    tvlct,255*[0,1,0,1,0,1,0,1],255*[0,0,1,1,0,0,1,1],255*[0,0,0,0,1,1,1,1]
  endelse
  !p.font = -1

; find borderline between expansion and collapse
; diag1,limitm1,limitl1

; find borderline between big bang and no big bang
; diag2,limitm2,limitl2

; load the pre-calculated, saved borderlines
  restore,file='diag.sav'

start:
  if not keyword_set(ps) then begin
    Device, Window_State=windowstate
    if windowstate[0] eq 0 then window,0 else wset,0
    aspect=float(!d.y_vsize)/float(!d.x_vsize)*3./4.
  endif

; start the plot, but don't draw anything yet -- we will draw the
; borders later, so they go on top of the hatchfill (looks nicer)
  plot,[0.,3.],[-1.,3.],xs=5,ys=5,/nodata,isotropic=isotropic,$
    xtitle='!4X!3!Dm!N',$
    ytitle='!4X!DK!3!N',charsize=chars

; draw border between expanding and collapsing
  xarea=[limitm1, 3., 0.,0.]
  yarea=[limitl1,-1.,-1.,0.]
  polyfill,xarea,yarea,/line_fill,orientation= 45.,spacing=0.5,lines=2,col=200
  oplot,limitm1,limitl1,lines=2

; draw border between big bang and no big bang
  xarea=[limitm2,0.,0.]
  yarea=[limitl2,3.,1.]
  polyfill,xarea,yarea,/line_fill,orientation=-45.,spacing=0.2,lines=1,col=200
  oplot,limitm2,limitl2,lines=2

; draw border between open and closed
; [to be completed]

; draw border between presently accelerating (q<0) and presently decelerating (q>0)
; [to be completed]

; indicate "our" universe (from exercise P5.3)
; [to be completed]

; annotation (orientation wants degrees, need to compensate for different window aspect ratios)
  xyouts,0.5,  2.5, 'No big bang',      align=0.5,charsize=chars,orient=atan(0.6 *aspect)*180./!pi
  xyouts,0.7,  2.1, 'loitering',        align=0.5,charsize=chars,orient=atan(1.4 *aspect)*180./!pi
  xyouts,1.9,  0.12,'eternal expansion',align=0.5,charsize=chars,orient=atan(0.09*aspect)*180./!pi
  xyouts,1.93,-0.10,'final recollapse', align=0.5,charsize=chars,orient=atan(0.09*aspect)*180./!pi
; [to be completed]

; now draw the plot axes & borders on top of everything
  plot,[0.,3.],[-1.,3.],xs=1,ys=1,/nodata,isotropic=isotropic,$
    /noerase,yticks=8,yminor=5,$
    xtitle='!4X!3!Dm!N',$
    ytitle='!4X!DK!3!N',charsize=chars

  if keyword_set(ps) then begin
    device,/close
    set_plot,'x'
    return
  endif

; prepare online calculation at desired position in om-ol diagram
  print,'to finish, click at left border outside plot'
  print,'otherwise, click at desired position in omega diagram'
  print
  xhair,om,ol
  if om lt -0.1 then return
  om=max([ 0.,om])
  om=min([ 3.,om])
  ol=max([-1.,ol])
  ol=min([ 3.,ol])
  print,' Omega_mat = ',om,' Omega_lam = ',ol

; define tmax for calculation
  if (om gt 1. and ol lt 0.3) then begin
; region around border expansion/collapse
    tm=12.
  endif else begin
    if ol lt 0. and ol gt -0.4 then begin
; collapsing models with long tmax
      tm=12.
    endif else begin
      tm=3.
    endelse
  endelse

; prepare input, perform calculation and read output
  openw,1,'in.idl'
  printf,1,tm
; set omega_rad = 0
  printf,1,om,', 0., ',ol
  close,1
; bs is executable for Burlisch-Stoer solution of Friedmann equation
  spawn,'./bs <in.idl >out.idl'
  readcol,'FRIEDMANN.OUT',t,a,/silent,skipline=1

  ym=max(a)
  ym=min([ym,10.])
  tm=min([tm,max(t)])

  Device, Window_State=windowstate
  if windowstate[1] eq 0 then window,1 else wset,1

  plot,t,a,xrange=[min(t),tm],yrange=[0,ym],$
    xtitle='t/H!D0!N',ytitle='a(t)',charsize=chars
  plots,[min(t),tm],[1.,1.],lines=1
  plots,[0.,0.],[0.,ym],lines=1
  xyouts,0.20,0.89,'!4X!3!Dm!N = ',           align=1.,charsize=chars,/normal
  xyouts,0.20,0.89,string(om,format='(f6.3)'),align=0.,charsize=chars,/normal
  xyouts,0.20,0.85,'!4X!DK!3!N = ',           align=1.,charsize=chars,/normal
  xyouts,0.20,0.85,string(ol,format='(f6.3)'),align=0.,charsize=chars,/normal

  goto,start

end

